import { Event } from '../../../../base/common/event.js';
import { IDisposable } from '../../../../base/common/lifecycle.js';
import { IView } from '../../../../base/browser/ui/grid/grid.js';
import '../../../../editor/editor.api.js';

interface ITitleVariable {
    readonly name: string;
    readonly contextKey: string;
}
interface ITitleProperties {
    isPure?: boolean;
    isAdmin?: boolean;
    prefix?: string;
}
interface ITitlebarPart extends IDisposable {
    /**
     * An event when the menubar visibility changes.
     */
    readonly onMenubarVisibilityChange: Event<boolean>;
    /**
     * Update some environmental title properties.
     */
    updateProperties(properties: ITitleProperties): void;
    /**
     * Adds variables to be supported in the window title.
     */
    registerVariables(variables: ITitleVariable[]): void;
}
interface IAuxiliaryTitlebarPart extends ITitlebarPart, IView {
    readonly container: HTMLElement;
    readonly height: number;
}

export type { IAuxiliaryTitlebarPart, ITitleProperties, ITitleVariable, ITitlebarPart };
